// AppTechObjectPool.hxx
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(APPTECHOBJECTPOOL_HXX__INCLUDED) // {
#define APPTECHOBJECTPOOL_HXX__INCLUDED

#include "Config.hxx"
#include "ObjectPool.hxx"
#include "AppTech.hxx"
#include "AppTechFlammableBuilding.hxx"

namespace Rescue
{
    /////////////////////////////////////////////////////////////////////////
    // AppTechObjectPool

    class Input;
    class Output;
    /** Gathering of object is managed */
    class AppTechObjectPool : public ObjectPool
    {
    private:
        
    protected:
#if 0
        Objects m_unregisterdBuilding;
#endif
    public:
        virtual ~AppTechObjectPool();
        AppTechObjectPool(const Config&);
        AppTechObjectPool();

        virtual Object* newObject(TypeId type);

#if 0
        /**
        * Object is registered.
        * @param    newObject
        * The object which is registered newly. This object delete is done by ObjectPool. Id it is the setting being completed.
        */
        virtual void add(Object* newObject);
#endif

        void dataread();
        void mkdat(S32 time);
        
        void fireInit(S32 time);
        void fireUpdate(S32 time);
        void step(S32 time);
    public:

  // Spread of fire parameter
        class fprmt {
        public:
            double  tsl[3]; // Burning falling time (woodwork, fire prevention and fire-resistance)
            double  ts1[3]; // Flame blowing out time (woodwork, fire prevention and fire-resistance)
            double  prc;    // Time ratio C to mature stage 0.4
            double  pra;    // Tilted factor 8 of å function
            double  prv1;   // Wind velocity correction coefficient -0.4
            double  cww[3]; // Weight of classified by building structure
            double  sta1;   // Standard 1F area 100.0m2
            double  stal;   // Standard preparing deck area 100.0m2
            double  stf1;   // Standard 1F building height 3m
            double  stqw;   // Standard fire load 120.0kg/m2
            double  sts;    // Standard prospect angle 2.94
            double  std;    // Standard distance 1.0m
            double  ctw[3]; // Wooden building ignition reference level 
       //(cumulative spread of fire influence quantity, 
       //unit time spread of fire influence quantity and 
       //unit time maximum spread of fire influence quantity)

            S32 dt; //  T time it cuts, (second)
            S32 itoukai;// Collapse flag (1: it considers 2: It does not consider)

            fprmt() {
            }
            fprmt(const Config& cf){
                tsl[0] = cf.TSL1(); // Burning falling time (woodwork, fire prevention and fire-resistance)
                tsl[1] = cf.TSL2(); // Burning falling time (woodwork, fire prevention and fire-resistance)
                tsl[2] = cf.TSL3(); // Burning falling time (woodwork, fire prevention and fire-resistance)
                ts1[0] = cf.TS11(); // Flame blowing out time (woodwork, fire prevention and fire-resistance)
                ts1[1] = cf.TS12(); // Flame blowing out time (woodwork, fire prevention and fire-resistance)
                ts1[2] = cf.TS13(); // Flame blowing out time (woodwork, fire prevention and fire-resistance)
                prc = cf.PRC();     // Time ratio C to mature stage 0.4
                pra = cf.PRA();     // Tilted factor 8 of å function
                prv1 = cf.PRV1();   // Wind velocity correction coefficient -0.4
                cww[0] = cf.CWW1(); // Weight of classified by building structure
                cww[1] = cf.CWW2(); // Weight of classified by building structure
                cww[2] = cf.CWW3(); // Weight of classified by building structure
                sta1 = cf.STA1();   // Standard 1F area 100.0m2
                stal = cf.STAL();   // Standard preparing deck area 100.0m2
                stf1 = cf.STF1();   // Standard 1F building height 3m
                stqw = cf.STQW();   // Standard fire load 120.0kg/m2
                sts = cf.STS();     // Standard prospect angle 2.94
                std = cf.STD();     // Standard distance 1.0m
                ctw[0] = cf.CTW1(); // Wooden building ignition reference level 
                ctw[1] = cf.CTW2(); // Wooden building ignition reference level 
                ctw[2] = cf.CTW3(); // Wooden building ignition reference level 
       //(cumulative spread of fire influence quantity, 
       //unit time spread of fire influence quantity and 
       //unit time maximum spread of fire influence quantity)

                dt = cf.DT();   //  T time it cuts, (second)
                //S32   itoukai;// Collapse flag (1: it considers 2: It does not consider)
            }
        };
        fprmt para;

        class tutu_n {
        public:
            // target building
            AppTechFlammableBuilding* building;
            S32 num; // XXX: target_buildin_id 

            S32 direction;
            XY xy;
            double quantity;
            S32 ewsn;
            
            tutu_n(AppTechFlammableBuilding* fb, S32 dir, S32 x, S32 y, S32 quantity, bool round_quantity) {
                building = fb;
                direction = dir;
                xy = XY(x, y);
                if (round_quantity)
                    this->quantity = quantity / 1000; // 1/1000 m3/min -> m3/min 
                else
                    this->quantity = quantity / 1000.0; // 1/1000 m3/min -> m3/min 
            }
        };
        std::vector<tutu_n> in_tutu;
#if 1
        std::vector<AppTechFlammableBuilding*> fb_v;
#else
        AppTechFlammableBuilding* tikudat[1024];
        int tikudat_size;
#endif

        class dummy_array {
        public:
            int size() { return objpool->fb_v.size(); }
            AppTechObjectPool* objpool;
            dummy_array(AppTechObjectPool* op = NULL){
                objpool = op;
            }
        };

        class dummy_tikudat :public dummy_array{
        public:
            dummy_tikudat(AppTechObjectPool* op = NULL) :dummy_array(op) {}
            AppTechFlammableBuilding& operator[](int i) { return *(objpool->fb_v[i]); }
        } tikudat;

        class dummy_rindat :public dummy_array{
        public:
            dummy_rindat(AppTechObjectPool* op = NULL) :dummy_array(op) {}
            AppTechFlammableBuilding::rinsetu_v& operator[](int i) { return (*(objpool->fb_v[i])).nrinsetu; }
        } rindat;

        class dummy_fire :public dummy_array{
        public:
            dummy_fire(AppTechObjectPool* op = NULL) :dummy_array(op) {}
            AppTechFlammableBuilding::ensho& operator[](int i) { return (*(objpool->fb_v[i])).fire; }
        } fire;

        class dummy_now :public dummy_array{
        public:
            dummy_now(AppTechObjectPool* op = NULL) :dummy_array(op) {}
            AppTechFlammableBuilding::tmp& operator[](int i) { return (*(objpool->fb_v[i])).now; }
        } now;

        class dummy_enslog :public dummy_array{
        public:
            dummy_enslog(AppTechObjectPool* op = NULL) :dummy_array(op) {}
            AppTechFlammableBuilding::log& operator[](int i) { return (*(objpool->fb_v[i])).fire_log; }
        } enslog;

        int hensu_MP_nkk; // XXX: Building number of cases

//      int icnt;
        int icall;
        int tnow;
//      int icall_tmp;
//      int tnow_tmp;

        class ks {
        public:
            int at_ks_ndw;      //! Wind direction (second). Y-axis positive sense 0 DEG with half clockwise.
            double at_ks_wvv;   //! Wind velocity
            ks(){
                at_ks_ndw = 0;
                at_ks_wvv = 0;
            }
        };
        ks kisyo;

        void calc(const S32 time);
    private:
        AppTechObjectPool(const AppTechObjectPool& source);
        AppTechObjectPool& operator= (const AppTechObjectPool& rhs);
        bool operator== (const AppTechObjectPool& rhs) const;
    };
    
    /////////////////////////////////////////////////////////////////////////
} // namespace Rescue

#ifdef NDEBUG
#  include "AppTechObjectPool.inl"
#endif

#endif // } !defined(APPTECHOBJECTPOOL_HXX__INCLUDED)

